@@ -6,7 +6,6 @@
 
 #include "barcodequickitem.h"
 
-#include <QDebug>
 #include <QGuiApplication>
 #include <QPainter>
 #include <QScreen>
@@ -20,30 +19,45 @@ BarcodeQuickItem::BarcodeQuickItem(QQuickItem *parent)
 
 BarcodeQuickItem::~BarcodeQuickItem() = default;
 
-QString BarcodeQuickItem::content() const
+QVariant BarcodeQuickItem::content() const
 {
     return m_content;
 }
 
-void BarcodeQuickItem::setContent(const QString &content)
+void BarcodeQuickItem::setContent(const QVariant &content)
 {
-    if (m_content == content)
+    if (m_content == content) {
         return;
+    }
     m_content = content;
     Q_EMIT contentChanged();
     updateBarcode();
 }
 
-BarcodeQuickItem::BarcodeType BarcodeQuickItem::barcodeType() const
+QJSValue BarcodeQuickItem::barcodeType() const
 {
-    return static_cast<BarcodeType>(m_type);
+    if (m_type) {
+        return static_cast<BarcodeType>(m_type.value());
+    }
+    return QJSValue();
 }
 
-void BarcodeQuickItem::setBarcodeType(BarcodeQuickItem::BarcodeType type)
+void BarcodeQuickItem::setBarcodeType(const QJSValue &type)
 {
-    if (m_type == static_cast<Prison::BarcodeType>(type))
-        return;
-    m_type = static_cast<Prison::BarcodeType>(type);
+    if (!type.isNumber()) {
+        if (m_type) {
+            m_type.reset();
+        } else {
+            return;
+        }
+
+    } else {
+        auto enumType = static_cast<Prison::BarcodeType>(type.toInt());
+        if (enumType == m_type) {
+            return;
+        }
+        m_type = enumType;
+    }
     Q_EMIT barcodeTypeChanged();
     m_barcode.reset();
     updateBarcode();
@@ -56,8 +70,9 @@ QColor BarcodeQuickItem::foregroundColor() const
 
 void BarcodeQuickItem::setForegroundColor(const QColor &color)
 {
-    if (m_fgColor == color)
+    if (m_fgColor == color) {
         return;
+    }
     m_fgColor = color;
     Q_EMIT foregroundColorChanged();
     updateBarcode();
@@ -70,8 +85,9 @@ QColor BarcodeQuickItem::backgroundColor() const
 
 void BarcodeQuickItem::setBackgroundColor(const QColor &color)
 {
-    if (m_bgColor == color)
+    if (m_bgColor == color) {
         return;
+    }
     m_bgColor = color;
     Q_EMIT backgroundColorChanged();
     updateBarcode();
@@ -79,16 +95,19 @@ void BarcodeQuickItem::setBackgroundColor(const QColor &color)
 
 BarcodeQuickItem::Dimensions Prison::BarcodeQuickItem::dimensions() const
 {
-    return m_barcode ? static_cast<BarcodeQuickItem::Dimensions>(m_barcode->dimensions()) : NoDimensions;
+    if (m_barcode)
+        return static_cast<BarcodeQuickItem::Dimensions>(m_barcode->dimensions());
+    return BarcodeQuickItem::Dimensions::NoDimensions;
 }
 
 void BarcodeQuickItem::paint(QPainter *painter)
 {
-    if (!m_barcode)
+    if (!m_barcode) {
         return;
+    }
 
-    const auto w_max = std::max(implicitWidth(), width());
-    const auto h_max = std::max(implicitHeight(), height());
+    const auto w_max = std::max(minimumWidth(), width());
+    const auto h_max = std::max(minimumHeight(), height());
     const auto img = m_barcode->toImage(QSizeF(w_max, h_max));
     const auto x = (w_max - img.width()) / 2;
     const auto y = (h_max - img.height()) / 2;
@@ -104,36 +123,58 @@ void BarcodeQuickItem::componentComplete()
 
 qreal BarcodeQuickItem::minimumHeight() const
 {
-    return m_barcode ? m_barcode->trueMinimumSize().height() : 0.0;
+    return m_barcode ? m_barcode->minimumSize().height() : 0;
 }
 
 qreal BarcodeQuickItem::minimumWidth() const
 {
-    return m_barcode ? m_barcode->trueMinimumSize().width() : 0.0;
+    return m_barcode ? m_barcode->minimumSize().width() : 0;
+}
+
+bool BarcodeQuickItem::isEmpty() const
+{
+    switch (m_content.userType()) {
+    case QMetaType::QString:
+        return m_content.toString().isEmpty();
+    case QMetaType::QByteArray:
+        return m_content.toByteArray().isEmpty();
+    default:
+        break;
+    }
+    return true;
 }
 
 void BarcodeQuickItem::updateBarcode()
 {
-    if (!isComponentComplete())
+    if (!isComponentComplete()) {
         return;
+    }
 
-    if (m_type == Prison::Null || m_content.isEmpty()) {
+    if (isEmpty() || !m_type) {
         m_barcode.reset();
         update();
         Q_EMIT dimensionsChanged();
         return;
     }
+    if (!m_barcode) {
+        m_barcode = Prison::Barcode::create(m_type.value());
+    }
+    if (!m_barcode) {
+        return;
+    }
 
-    if (!m_barcode)
-        m_barcode.reset(Prison::createBarcode(m_type));
-    if (m_barcode) {
-        m_barcode->setData(m_content);
+        if (m_content.userType() == QMetaType::QString) {
+            m_barcode->setData(m_content.toString());
+        } else {
+            m_barcode->setData(m_content.toByteArray());
+        }
         m_barcode->setForegroundColor(m_fgColor);
         m_barcode->setBackgroundColor(m_bgColor);
         const auto size = m_barcode->preferredSize(QGuiApplication::primaryScreen()->devicePixelRatio());
         setImplicitSize(size.width(), size.height());
-    }
 
-    update();
-    Q_EMIT dimensionsChanged();
+        update();
+        Q_EMIT dimensionsChanged();
 }
+
+#include "moc_barcodequickitem.cpp"
